package PocketIO::Message;

use strict;
use warnings;

use base 'Protocol::SocketIO::Message';

1;
__END__

=head1 NAME

PocketIO::Message - Socket.IO message parsing and building

=head1 DESCRIPTION

L<PocketIO::Message> parsers and builds Socket.IO messages.

=cut
