package PocketIO::Transport::XHRPolling;

use strict;
use warnings;

use base 'PocketIO::Transport::BasePolling';

1;
__END__

=head1 NAME

PocketIO::Transport::XHRPolling - XHRPolling transport

=head1 DESCRIPTION

L<PocketIO::Transport::XHRPolling> is a C<xhr-polling> transport implementation.

=head1 METHODS

Inherict all methods from L<PocketIO::Transport::BasePolling>.

=cut
